-----------------------------------------------------------
--|> CORE
-----------------------------------------------------------
local ADDON_NAME, NS = ...

--|> Upvalue Globals
-----------------------------------------------------------
local MAX_PLAYER_LEVEL = MAX_PLAYER_LEVEL
local GetUnitName = GetUnitName
local GetRealmName = GetRealmName
local UnitFactionGroup = UnitFactionGroup
local CopyTable = CopyTable
local GetScreenWidth = GetScreenWidth
local GetScreenHeight = GetScreenHeight
local GetAddOnMetadata = GetAddOnMetadata
local select = select

weizPVP.enabled = false

--|> Addon Global Controls
-----------------------------------------------------------
--> Enable
function weizPVP:OnEnable()
    if not weizPVP.enabled then
        weizPVP.enabled = true
        self:CancelAllTimers()
        self:ScheduleRepeatingTimer(NS.PulseEvent, 1)
        NS.Options.Addon.Enabled = true
        NS.SetWindowSettings()
        NS.EnableEvents()
    end
end

--> Disable
function weizPVP:OnDisable()
    if weizPVP.enabled then
        weizPVP.enabled = false
        self:CancelAllTimers()
        NS.DisableEvents()
        NS.ClearListData()
        NS.Options.Addon.Enabled = false
        NS.SetWindowSettings()
    end
end

--> Clear List Data
function weizPVP.ClearListData()
    NS.ClearListData()
end

--> Toggle Window
function weizPVP.ToggleWindows()
    if NS.Options.Addon.Enabled then
        NS.Options.Window.Visible = not NS.Options.Window.Visible
        NS.SetWindowSettings()
    end
end

--> Toggle Crosshair
function weizPVP.ToggleCrosshair()
    NS.Crosshair.Toggle()
end

--> Get Player Info
function NS.GetPlayerInfo()
    --> Player Info
    NS.Player = {}
    NS.Player.GUID = UnitGUID("player")
    NS.Player.Name = GetUnitName("player")
    NS.Player.Faction = select(1, UnitFactionGroup("player"))
    NS.Player.Level = UnitLevel("player")
    NS.Player.FromRealm = GetRealmName()
    NS.Player.FromSubRealm = gsub(NS.Player.FromRealm, "[%s%-]", "")
    NS.PlayerRealm = NS.Player.FromSubRealm
end

--> Get Game Info
function NS.GetGameInfo()
    --> Game Info
    NS.Game = {}
    NS.Game.MaxPlayerLevel = MAX_PLAYER_LEVEL
end

--> Get Display Info
function NS.GetDisplayInfo()
    NS.Display = {}
    NS.Display.Width = GetScreenWidth()
    NS.Display.Height = GetScreenHeight()
    NS.Display.xFactor = (768 / NS.Display.Width) * (NS.Display.Width / NS.Display.Height)
    NS.Display.yFactor = 768 / NS.Display.Height
    NS.Display.EffectiveScale = UIParent:GetEffectiveScale()
    NS.Display.CVarUiScale = GetCVar("uiScale")
end

--> Print Addon Message
function NS.PrintAddonMessage(msg)
    (SELECTED_CHAT_FRAME or DEFAULT_CHAT_FRAME):AddMessage(
        "|TInterface/Addons/weizPVP/Media/weizpvp_chat.tga:0|t " .. msg
    )
end

--> Print
function NS.Print(msg)
    NS.PrintAddonMessage(msg)
end

--> Update Addon Info
function NS.UpdateAddonInfo()
    weizPVP.Database_Version = 3
    weizPVP.Addon_Version = GetAddOnMetadata(ADDON_NAME, "Version")
end

--> Bindings
function NS.SetupBindings()
    _G.BINDING_HEADER_WEIZPVP = NS.Constants.AddonString
    _G.BINDING_NAME_WEIZPVPTOGGLE = "Toggle Window Visibility"
    _G.BINDING_NAME_WEIZPVPCLEARPLAYERLIST = "Clear Player List"
    _G.BINDING_NAME_WEIZPVPTOGGLECROSSHAIR = "Toggle Crosshair"
end

--> Reset All Customization
function NS.ResetAllCustomizations()
    NS.Options.Frames.Header = CopyTable(NS._DefaultOptions.profile.Options.Frames.Header)
    NS.Options.Frames.StatusPopUp = CopyTable(NS._DefaultOptions.profile.Options.Frames.StatusPopUp)
    NS.Options.Bars = CopyTable(NS._DefaultOptions.profile.Options.Bars)
    NS.Options.Frames.Scale = NS._DefaultOptions.profile.Options.Frames.Scale
    NS.Options.Frames.AutoScaleEnabled = NS._DefaultOptions.profile.Options.Frames.AutoScaleEnabled
    NS.Options.Frames.AutoScaleMultiplier = NS._DefaultOptions.profile.Options.Frames.AutoScaleMultiplier
    NS.Options.Frames.BackgroundColor = NS._DefaultOptions.profile.Options.Frames.BackgroundColor
    NS.CoreUI.Initialize()
end
